import React from 'react';
import ReactDOM from 'react-dom';
// import { Marker } from 'google-maps-react';

const mapStyles = {
    map: {
        position: 'absolute',
        width: '100%',
        height: '100%',
    }
};
export class CurrentLocation extends React.Component{
    
    constructor(props) {
        super(props);
        this.mapDOMRef = React.createRef();
        const { lat, lng } = this.props.initialCenter;
        this.state = {
          currentLocation: {
            lat: lat,
            lng: lng
          },
          Circles: {
            idCircle: null,
            mainCircle: null,
            odCircle: null
          },
          PlaceMarkers: [],
          SearchCirlces: []
        };
    }
    componentDidUpdate(prevProps, prevState){
        if(prevProps.google !== this.props.google){
            this.loadMap();
        }
        if(prevProps.dataFromParent !== this.props.dataFromParent){
          const distance = this.props.dataFromParent.Distance;
          const increment = this.props.dataFromParent.Increment;
          let latlng = {lat: this.state.currentLocation.lat, lng: this.state.currentLocation.lng};
          if(this.props.selectedMarker === 'currentLocation'){
            
            this.drawCircles(latlng, distance, increment, "#00dfff");
            this.searchPlaces(distance+increment, this.props.PlaceTypes, latlng, this.props.PlaceKeyword);
          }
          else{
            if(this.props.dynMarkerLocation.lat !== -1 || this.props.dynMarkerLocation.lng !== -1)
            //Marker has been moved - still at default location
            {
              latlng = {lat: this.props.dynMarkerLocation.lat, lng: this.props.dynMarkerLocation.lng};
            }
            this.drawCircles(latlng, distance, increment, "#ff8100");
            this.searchPlaces(distance+increment, this.props.PlaceTypes, latlng, this.props.PlaceKeyword);
          }
        }
        if(prevState.currentLocation !== this.state.currentLocation){
            this.recenterMap();
        }
    }
    recenterMap(){
        const map = this.map;
        const current = this.state.currentLocation;
        const google = this.props.google;
        const maps = google.maps;

        if(map){
            let center = new maps.LatLng(current.lat, current.lng);
            map.panTo(center);
        }
    }
    componentDidMount(){
        if(this.props.centerAroundCurrentLocation){
            if(navigator && navigator.geolocation){
                navigator.geolocation.getCurrentPosition(pos=> {
                    const coords = pos.coords;
                    this.setState({ 
                        currentLocation: {
                            lat: coords.latitude,
                            lng: coords.longitude
                        }
                    });
                });
                
            }
        }
        this.loadMap();
    }

    searchPlaces(radius, types, center, keyword) {
      // const center = {lat: lat, lng: lng}; //Current Location
      console.log(`searchPlaces: center: ${center}`);
      let request = {
        location: center,
        keyword: keyword,
        radius: radius,
        type: types,
      };
      const markers = this.state.PlaceMarkers;
      const maps = this.props.google.maps;
      const places = maps.places;
      const service = new places.PlacesService(this.map);
      const OK_STATUS = places.PlacesServiceStatus.OK;
      service.nearbySearch(request, (results, status, pagination) => {
        if (status === OK_STATUS && results) {
          console.log("results", results);
          for (const result of results) {
            const location = result.geometry.location;
            const lat = location.lat();
            const lng = location.lng();
            markers.push({
              place_id: result.place_id,
              icon: {
                url: result.icon,
                size: new maps.Size(71, 71),
                origin: new maps.Point(0, 0),
                anchor: new maps.Point(17, 34),
                scaledSize: new maps.Size(25, 25),
              },
              title: result.name,
              position: {lat: lat, lng: lng}
            });
          }  
          if(pagination && pagination.hasNextPage){
              pagination.nextPage();
          }
          else{
             this.props.handleMarkerChanges(markers); 
          }
        }
      });
    }

    drawCircles(center, Dist, Inc, color){
      const circleType = this.props.google.maps.Circle;        
      console.log('center',center, 'Dist', Dist,'Rad', Inc);
      const circles = this.state.Circles;
      // Set Inner Circle
      // if (circles.idCircle !== null) {
      //   circles.idCircle.setMap(null);
      // }
      circles.idCircle = createCircle(center, Dist-Inc);
      circles.idCircle.setMap(this.map);

      // Set Main Circle
      // if (circles.mainCircle !== null) {
      //   circles.mainCircle.setMap(null);
      // }
      circles.mainCircle = createCircle(center, Dist);
      circles.mainCircle.setMap(this.map);            
      
      // Set Outer Circle
      // if (circles.odCircle !== null) {
      //   circles.odCircle.setMap(null);
      // }
      circles.odCircle = createCircle(center, Dist+Inc);
      circles.odCircle.setMap(this.map);
      // Save the circles to state
      this.setState({ Circles: circles });

      function createCircle(center, radius) {
        console.log('drawCircles', radius);
        return new circleType({
          strokeColor: color,
          strokeOpacity: 0.8,
          strokeWeight: 2,
          fillColor: color,
          fillOpacity: 0.1,
          center: center,
          radius: radius
        });
      }
    }

    // drawCircles(center, Dist, Inc){
    //   //TODO Get Circles to Stay when Recalculated
    //   const circleType = this.props.google.maps.Circle;        
    //   console.log('center',center, 'Dist', Dist,'Rad', Inc);
    //   const circles = this.state.Circles;
    //   // Set Inner Circle
    //   if (circles.idCircle !== null) {
    //     circles.idCircle.setMap(null);
    //   }
    //   circles.push(createCircle(center, Dist-Inc));
    //   // circles.idCircle.setMap(this.map);

    //   // Set Main Circle
    //   if (circles.mainCircle !== null) {
    //     circles.mainCircle.setMap(null);
    //   }
    //   circles.mainCircle = createCircle(center, Dist);
    //   // circles.mainCircle.setMap(this.map);            
      
    //   // Set Outer Circle
    //   if (circles.odCircle !== null) {
    //     circles.odCircle.setMap(null);
    //   }
    //   circles.odCircle = createCircle(center, Dist+Inc);
    //   circles.odCircle.setMap(this.map);
    //   // Save the circles to state
    //   this.setState({ Circles: circles });

    //   function createCircle(center, radius) {
    //     console.log('drawCircles', radius);
    //     return new circleType({
    //       strokeColor: "#FF0000",
    //       strokeOpacity: 0.8,
    //       strokeWeight: 2,
    //       fillColor: "#FF0000",
    //       fillOpacity: 0.1,
    //       center: center,
    //       radius: radius
    //     });
    //   }
    // }

    loadMap(){
        if(this.props && this.props.google){
            const {google} = this.props;
            const maps = google.maps;
            const mapRef = this.mapDOMRef.current;  /*FIX ME*/

            //referencing the DOM element
            const node = ReactDOM.findDOMNode(mapRef);

            let {zoom} = this.props;
            const {lat, lng} = this.state.currentLocation;
            
            const center = new maps.LatLng(lat, lng);
            const MapConfig = Object.assign(
                {},
                {
                  center: center,
                  zoom: zoom,
                  disableDefaultUI: true,
                  streetViewControl: true,
                  zoomControl: true,
                  styles: [
                    {
                        featureType: "administrative",
                        stylers: [{ visibility: "on" }]
                    },
                    {
                      featureType: "poi",
                      stylers: [{ visibility: "off" }]
                    },
                    {
                      featureType: "transit",
                      stylers: [{ visibility: "off" }]
                    },
                    {
                      featureType: "landscape",
                      stylers: [{ visibility: "off" }]
                    },
                  ]
                }
            );
            this.map = new maps.Map(node, MapConfig);
        }
    }
    renderChildren() {
        const { children } = this.props;
    
        if (!children) return;
        return React.Children.map(children, c => {
          if (!c) return;
          return React.cloneElement(c, {
            map: this.map,
            google: this.props.google,
            mapCenter: this.state.currentLocation
          });
        });
    }


    render() {
    const style = Object.assign({}, mapStyles.map);  
    return (
        <>
        <div>
          <div style={style} ref={this.mapDOMRef}> {/*previously ref="map"*/}
            Loading map...
          </div>
          {this.renderChildren()}
        </div></>
      );
    }
}

CurrentLocation.defaultProps = {
    zoom: 14,
    initialCenter: {
      lat: 44.447540,
      lng: -88.070580
    },
    centerAroundCurrentLocation: false,
    visible: true
  };
export default CurrentLocation;