import React from 'react';
import ReactDOM from 'react-dom';

const mapStyles = {
    map: {
        position: 'absolute',
        width: '100%',
        height: '100%'
    }
};
export class CurrentLocation extends React.Component{
    constructor(props) {
        super(props);
    
        const { lat, lng } = this.props.initialCenter;
    
        this.state = {
          currentLocation: {
            lat: lat,
            lng: lng
          }
        };
    }
    componentDidUpdate(prevProps, prevState){
        if(prevProps.google !== this.props.google){
            this.loadMap();
        }
        if(prevState.currentLocation !== this.state.currentLocation){
            this.recenterMap();
        }
    }
    recenterMap(){
        const map = this.map;
        const current = this.state.currentLocation;
        const google = this.props.google;
        const maps = google.maps;

        if(map){
            let center = new maps.LatLng(current.lat, current.lng);
            map.panTo(center);
        }
    }
    componentDidMount(){
        if(this.props.centerAroundCurrentLocation){
            if(navigator && navigator.geolocation){
                navigator.geolocation.getCurrentPosition(pos=> {
                    const coords = pos.coords;
                    this.setState({
                        currentLocation: {
                            lat: coords.latitude,
                            lng: coords.longitude
                        }
                    });
                });
            }
        }
        this.loadMap();
    }
    loadMap(){
        if(this.props && this.props.google){
            const {google} = this.props;
            const maps = google.maps;
            const mapRef = this.refs.map;

            //referencing the DOM element
            const node = ReactDOM.findDOMNode(mapRef);

            let {zoom} = this.props;
            const {lat, lng} = this.state.currentLocation;
            const center = new maps.LatLng(lat, lng);

            const MapConfig = Object.assign(
                {},
                {
                    center: center,
                    zoom: zoom,
                    disableDefaultUI: true,
                    streetViewControl: true,
                    zoomControl: true
                    
                }
            );
            // const dataMenuDiv = document.createElement('div');
            // DataMenu(dataMenuDiv, this.map);
            // this.map.controls[google.maps.ControlPosition.TOP_LEFT].push(dataMenuDiv);
            //maps.Map() is constructor for map instiance
            this.map = new maps.Map(node, MapConfig);
        }
    }
    renderChildren() {
        const { children } = this.props;
    
        if (!children) return;
    
        return React.Children.map(children, c => {
          if (!c) return;
    
          return React.cloneElement(c, {
            map: this.map,
            google: this.props.google,
            mapCenter: this.state.currentLocation
          });
        });
    }
    render() {
    const style = Object.assign({}, mapStyles.map);
    return (
        <div>
          <div style={style} ref="map">
            Loading map...
          </div>
          {this.renderChildren()}
        </div>
      );
    }
}

// function DataMenu(controlDiv, map)
// {
// 	// Set CSS for the control border
// 	var controlUI = document.createElement('div');
// 	controlUI.style.backgroundColor = '#fff';
// 	controlUI.style.border = '2px solid #fff';
// 	controlUI.style.borderRadius = "3px"; //needed?
//   	controlUI.style.boxShadow = "0 2px 6px rgba(0,0,0,.3)"; //needed?
// 	controlUI.style.cursor = 'pointer';
// 	controlUI.style.marginBottom = '22px';
// 	controlUI.style.marginTop = '22px';
// 	controlUI.style.marginLeft = '22px';
// 	controlUI.style.marginRight = '22px';
// 	controlUI.style.textAlign = 'center';
// 	controlUI.title = 'Click to add data';
// 	controlDiv.appendChild(controlUI);
// 	// Set CSS for the control interior
// 	var controlText = document.createElement('div');
// 	controlText.style.color = 'rgb(25,25,25)';
// 	controlText.style.fontFamily = 'Roboto,Arial,sans-serif';
// 	controlText.style.fontSize = '16px';
// 	controlText.style.lineHeight = '38px';
// 	controlText.style.paddingLeft = '5px';
// 	controlText.style.paddingRight = '5px';
// 	controlText.innerHTML = 'Data Menu';
// 	controlUI.appendChild(controlText);
// 	// Setup the click event listener. 	This needs to open menu for adding data
// 	controlUI.addEventListener("click", () => {
// 		//open menu here
// 	  });
// }

CurrentLocation.defaultProps = {
    zoom: 14,
    initialCenter: {
      lat: 0,
      lng: 0
    },
    centerAroundCurrentLocation: false,
    visible: true
  };
export default CurrentLocation;